// Copyright (C) 2014 Ben Asselstine
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
//  02110-1301, USA.

#include <config.h>
#include <string>
#include "curlhelper.h"
#include <curlpp/Options.hpp>
#include <curlpp/Infos.hpp>

CurlHelper::CurlHelper(Glib::ustring url, Glib::ustring tmp)
{
  remove_temp = false;
  if (tmp == "")
    {
      int fd = Glib::file_open_tmp(tmpfile);
      close (fd);
      remove_temp = true;
    }
  else
    tmpfile = tmp;
  fileptr = fopen (tmpfile.c_str(), "wb");
  easy = new curlpp::Easy();
  easy->setOpt(curlpp::options::HttpGet(true));
  easy->setOpt(curlpp::options::Url(url));
  easy->setOpt(curlpp::options::WriteFile(fileptr));
}

CurlHelper::~CurlHelper()
{
  delete easy;
  if (remove_temp)
    remove (tmpfile.c_str());
}

bool CurlHelper::get()
{
  try
    {
      easy->perform();
    } catch (cURLpp::LogicError& exception) {
      remove (tmpfile.c_str());
      return false;
    } catch (cURLpp::RuntimeError& exception) {
      remove (tmpfile.c_str());
      return false;
    }
  fflush (fileptr);
  fsync (fileno (fileptr));
  fclose (fileptr);
  struct Msg msg;
  msg.easy = easy;
  std::string data = Glib::file_get_contents(tmpfile);
  msg.data = data.c_str();
  msg.response_code = curlpp::infos::ResponseCode().get(*easy);
  msg.len = data.length();
  return signal_recv.emit(msg);
}

void CurlHelper::download()
{
  try
    {
      easy->perform();
    } catch (cURLpp::LogicError& exception) {
      remove (tmpfile.c_str());
      return;
    } catch (cURLpp::RuntimeError& exception) {
      remove (tmpfile.c_str());
      return;
    }
  fflush (fileptr);
  fsync (fileno (fileptr));
  fclose (fileptr);
  int response_code = curlpp::infos::ResponseCode().get(*easy);
  return signal_down.emit(response_code, tmpfile);
}
