// Copyright (C) 2014 Ben Asselstine
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
//  02110-1301, USA.

#include <config.h>
#include <string>
#include "gunziphelper.h"

unsigned int GunzipHelper::chunk_size = 8192;
sigc::signal<void, Glib::ustring> GunzipHelper::signal;

GunzipHelper::GunzipHelper(Glib::ustring filename)
{
  infile = filename;
  std::string prefix = PACKAGE;
  int fd = Glib::file_open_tmp (outfile, prefix);
  close(fd);
}

GunzipHelper::~GunzipHelper()
{
}

void GunzipHelper::on_stream_ready(Glib::RefPtr<Gio::AsyncResult>& result, Glib::RefPtr<Gio::FileInputStream> &s, Glib::RefPtr<Gio::ZlibDecompressor> gz, Glib::RefPtr<Gio::File> &out)
{
  Glib::RefPtr<Glib::Bytes> bytes = s->read_bytes_finish(result);
  void *outbuf = malloc(chunk_size * 100);
  gsize bytesread = 0;
  gsize byteswritten = 0;
  gsize bufsize = 0;
  gz->convert(bytes->get_data(bufsize), bytes->get_size(), 
              outbuf, chunk_size *100, Gio::CONVERTER_NO_FLAGS, 
              bytesread, byteswritten);
  out->append_to()->write((const char*) outbuf, byteswritten);
  free (outbuf);
  if (bytes->get_size() == chunk_size)
    {
      Glib::RefPtr<Gio::Cancellable> cancel = Gio::Cancellable::create();
      s->read_bytes_async(chunk_size, sigc::bind(sigc::mem_fun(*this, &GunzipHelper::on_stream_ready), s, gz, out), cancel);
    }
  else
    signal(out->get_path());
}

void GunzipHelper::on_file_ready(Glib::RefPtr<Gio::AsyncResult>& result, Glib::RefPtr<Gio::File> &f, Glib::RefPtr<Gio::File> &out)
{
  Glib::RefPtr<Gio::Cancellable> cancel = Gio::Cancellable::create();
  Glib::RefPtr<Gio::FileInputStream> s = f->read_finish(result);
  Glib::RefPtr<Gio::ZlibDecompressor > gz = Gio::ZlibDecompressor::create(Gio::ZLIB_COMPRESSOR_FORMAT_GZIP);
  s->read_bytes_async(chunk_size, sigc::bind(sigc::mem_fun(*this, &GunzipHelper::on_stream_ready), s, gz, out), cancel);
}

void GunzipHelper::gunzip()
{
  Glib::RefPtr<Gio::File> f = Gio::File::create_for_path(infile);
  Glib::RefPtr<Gio::File> out = Gio::File::create_for_path(outfile);
  Glib::RefPtr<Gio::Cancellable> cancel = Gio::Cancellable::create();
  f->read_async(sigc::bind(sigc::mem_fun(this, &GunzipHelper::on_file_ready), f, out), cancel);
}
