//  Copyright (C) 2011, 2012, 2014 Ben Asselstine
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
//  02110-1301, USA.

#include <config.h>
#include "jamendo-track-list-loader.h"
#include "track-list.h"
#include "track.h"
#include "ucompose.hpp"
#include <zlib.h>
#include <cstdio>
#include <libxml/tree.h>
#include "gunziphelper.h"


JamendoTrackListLoader::JamendoTrackListLoader()
{
}

JamendoTrackListLoader::~JamendoTrackListLoader()
{
}

Glib::ustring JamendoTrackListLoader::lookup_genre(int genre_id)
{
  switch (genre_id)
    {
    case 0: return "Blues"; case 1: return "Classic Rock";
    case 2: return "Country"; case 3: return "Dance";
    case 4: return "Disco"; case 5: return "Funk";
    case 6: return "Grunge"; case 7: return "Hip-Hop";
    case 8: return "Jazz"; case 9: return "Metal";
    case 10: return "New Age"; case 11: return "Oldies";
    case 12: return "Other"; case 13: return "Pop";
    case 14: return "R&B"; case 15: return "Rap";
    case 16: return "Reggae"; case 17: return "Rock";
    case 18: return "Techno"; case 19: return "Industrial";
    case 20: return "Alternative"; case 21: return "Ska";
    case 22: return "Death Metal"; case 23: return "Pranks";
    case 24: return "Soundtrack"; case 25: return "Euro-Techno";
    case 26: return "Ambient"; case 27: return "Trip-Hop";
    case 28: return "Vocal"; case 29: return "Jazz+Funk";
    case 30: return "Fusion"; case 31: return "Trance";
    case 32: return "Classical"; case 33: return "Instrumental";
    case 34: return "Acid"; case 35: return "House";
    case 36: return "Game"; case 37: return "Sound Clip";
    case 38: return "Gospel"; case 39: return "Noise";
    case 40: return "AlternRock"; case 41: return "Bass";
    case 42: return "Soul"; case 43: return "Punk";
    case 44: return "Space"; case 45: return "Meditative";
    case 46: return "Instrumental Pop"; case 47: return "Instrumental Rock";
    case 48: return "Ethnic"; case 49: return "Gothic";
    case 50: return "Darkwave"; case 51: return "Techno-Industrial";
    case 52: return "Electronic"; case 53: return "Pop-Folk ";
    case 54: return "Eurodance"; case 55: return "Dream";
    case 56: return "Southern Rock"; case 57: return "Comedy";
    case 58: return "Cult"; case 59: return "Gangsta Rap";
    case 60: return "Top 40"; case 61: return "Christian Rap";
    case 62: return "Pop / Funk"; case 63: return "Jungle";
    case 64: return "Native American"; case 65: return "Cabaret";
    case 66: return "New Wave"; case 67: return "Psychedelic";
    case 68: return "Rave"; case 69: return "Showtunes";
    case 70: return "Trailer"; case 71: return "Lo-Fi";
    case 72: return "Tribal"; case 73: return "Acid Punk";
    case 74: return "Acid Jazz"; case 75: return "Polka";
    case 76: return "Retro"; case 77: return "Musical";
    case 78: return "Rock & Roll"; case 79: return "Hard Rock";
    case 80: return "Folk"; case 81: return "Folk-Rock";
    case 82: return "National Folk"; case 83: return "Swing";
    case 84: return "Fast  Fusion"; case 85: return "Bebob";
    case 86: return "Latin"; case 87: return "Revival";
    case 88: return "Celtic"; case 89: return "Bluegrass";
    case 90: return "Avantgarde"; case 91: return "Gothic Rock";
    case 92: return "Progressive Rock"; case 93: return "Psychedelic Rock";
    case 94: return "Symphonic Rock"; case 95: return "Slow Rock";
    case 96: return "Big Band"; case 97: return "Chorus";
    case 98: return "Easy Listening"; case 99: return "Acoustic";
    case 100: return "Humour"; case 101: return "Speech";
    case 102: return "Chanson"; case 103: return "Opera";
    case 104: return "Chamber Music"; case 105: return "Sonata";
    case 106: return "Symphony"; case 107: return "Booty Bass";
    case 108: return "Primus"; case 109: return "Porn Groove";
    case 110: return "Satire"; case 111: return "Slow Jam";
    case 112: return "Club"; case 113: return "Tango";
    case 114: return "Samba"; case 115: return "Folklore";
    case 116: return "Ballad"; case 117: return "Power Ballad";
    case 118: return "Rhythmic Soul"; case 119: return "Freestyle";
    case 120: return "Duet"; case 121: return "Punk Rock";
    case 122: return "Drum Solo"; case 123: return "A Cappella";
    case 124: return "Euro-House"; case 125: return "Dance Hall";
    case 126: return "Goa"; case 127: return "Drum & Bass";
    case 128: return "Club-House"; case 129: return "Hardcore";
    case 130: return "Terror"; case 131: return "Indie";
    case 132: return "BritPop"; case 133: return "Negerpunk";
    case 134: return "Polsk Punk"; case 135: return "Beat";
    case 136: return "Christian Gangsta Rap"; case 137: return "Heavy Metal";
    case 138: return "Black Metal"; case 139: return "Crossover";
    case 140: return "Contemporary Christian"; 
    case 141: return "Christian Rock";
    case 142: return "Merengue"; case 143: return "Salsa";
    case 144: return "Thrash Metal"; case 145: return "Anime";
    case 146: return "JPop"; case 147: return "Synthpop";
    case 148: return "Rock/Pop"; 
    }

  return "";
}

void JamendoTrackListLoader::parse_tags(xmlNodePtr child, Glib::ustring &genre)
{
  xmlNodePtr tags = child->xmlChildrenNode;
  while (tags)
    {
      xmlChar* value;
      xmlNodePtr tag_property = tags->xmlChildrenNode;
      if (xmlStrcasecmp(tags->name, BAD_CAST "tag") == 0) 
        {
          value = xmlNodeGetContent(tag_property);
          Glib::ustring 
            strvalue( reinterpret_cast<const char*>(value));
          if (genre.empty() == true)
            genre = strvalue;
          else
            genre += ", " +strvalue;
          xmlFree (value);
        }
      tags = tags->next;
    }
}
void JamendoTrackListLoader::parse_tracks(xmlNodePtr child, TrackList *ts, Glib::ustring artist_name)
{
  xmlNodePtr tracks = child->xmlChildrenNode;
  while (tracks)
    {
      if (xmlStrcasecmp(tracks->name, BAD_CAST "track") == 0) 
        {
          Glib::ustring tag_genres;
          xmlChar* value;
          xmlNodePtr track_property = tracks->xmlChildrenNode;
          double duration = 0.0;
          Glib::ustring track_id;
          int genre_id = 0;
          Glib::ustring track_name;
          Glib::ustring license_url;
          while (track_property)
            {
              if (xmlStrcasecmp(track_property->name, BAD_CAST "id") == 0) 
                {
                  value = xmlNodeGetContent(track_property);
                  Glib::ustring 
                    strvalue( reinterpret_cast<const char*>(value));
                  track_id = strvalue;
                  xmlFree (value);
                }
              else if (xmlStrcasecmp(track_property->name, BAD_CAST "id3genre") == 0) 
                {
                  value = xmlNodeGetContent(track_property);
                  Glib::ustring 
                    strvalue( reinterpret_cast<const char*>(value));
                  genre_id = atoi(strvalue.c_str());
                  xmlFree (value);
                }
              else if (xmlStrcasecmp(track_property->name, BAD_CAST "license") == 0) 
                {
                  value = xmlNodeGetContent(track_property);
                  Glib::ustring 
                    strvalue( reinterpret_cast<const char*>(value));
                  license_url = strvalue;
                  xmlFree (value);
                }
              else if (xmlStrcasecmp(track_property->name, BAD_CAST "duration") == 0) 
                {
                  value = xmlNodeGetContent(track_property);
                  Glib::ustring 
                    strvalue( reinterpret_cast<const char*>(value));
                  duration = atof(strvalue.c_str());
                  xmlFree (value);
                }
              else if (xmlStrcasecmp(track_property->name, BAD_CAST "name") == 0) 
                {
                  value = xmlNodeGetContent(track_property);
                  Glib::ustring 
                    strvalue( reinterpret_cast<const char*>(value));
                  track_name = strvalue;
                  xmlFree (value);
                }
              else if (xmlStrcasecmp(track_property->name, BAD_CAST "Tags") == 0)
                {
                  parse_tags(track_property, tag_genres);
                }
              track_property = track_property->next;
            }
          Glib::ustring lic;
          
          Glib::ustring g = tag_genres;
          if (tag_genres.empty() == true)
            {
              if (genre_id > 0)
                g = lookup_genre(genre_id);
              else
                g = "Unknown";            }
          Track* t = new Track(duration, g, track_name, artist_name, track_id, 
             String::ucompose("http://api.jamendo.com/get2/stream/track/redirect/?id=%1&streamencoding=mp31", track_id),
                               String::ucompose("http://www.jamendo.com/en/track/%1", track_id), get_license_name(license_url.c_str()), 0.0);
          ts->push_back(t);
        }
      tracks = tracks->next;
    }
}

void JamendoTrackListLoader::parse_albums(xmlNodePtr child, TrackList *tracklist, Glib::ustring artist_name)
{
  xmlNodePtr albums = child->xmlChildrenNode;
  while (albums)
    {
      if (xmlStrcasecmp(albums->name, BAD_CAST "album") == 0) 
        {
          xmlNodePtr album_property = albums->xmlChildrenNode;
          while (album_property)
            {
              if (xmlStrcasecmp(album_property->name, BAD_CAST "Tracks") == 0)
                parse_tracks(album_property, tracklist, artist_name);
              album_property = album_property->next;
            }
        }
      albums = albums->next;
    }
}

void JamendoTrackListLoader::parse(xmlNodePtr child, TrackList *tracklist)
{
  xmlNodePtr artists = child->xmlChildrenNode;
  while (artists)
    {
      if (xmlStrcasecmp(artists->name, BAD_CAST "artist") == 0) 
        {
          xmlChar* value;
          Glib::ustring artist_name;
          Glib::ustring genre;
          xmlNodePtr artist_property = artists->xmlChildrenNode;
          while (artist_property)
            {
              if (xmlStrcasecmp(artist_property->name, BAD_CAST "name") == 0) 
                {
                  value = xmlNodeGetContent(artist_property);
                  Glib::ustring 
                    strvalue( reinterpret_cast<const char*>(value));
                  artist_name = strvalue;
                  xmlFree (value);
                }
              else if (xmlStrcasecmp(artist_property->name, BAD_CAST "Albums") == 0) 
                {
                  parse_albums(artist_property, tracklist, artist_name);
                }

              artist_property = artist_property->next;
            }
        }
      artists = artists->next;
    }
}

TrackList* JamendoTrackListLoader::parse(const char *filename)
{
  xmlDocPtr doc;
  xmlNodePtr root;

  doc = xmlParseFile(filename);
  if (!doc)
    return NULL;

  root = xmlDocGetRootElement(doc);

  if (!root)
    return NULL;
  xmlNodePtr child = root->xmlChildrenNode;

  std::map<guint32, Glib::ustring> artists;
  std::map<guint32, Glib::ustring> genres;
  std::map<guint32, guint32> albums;
  std::map<guint32, double> album_ratings;

  TrackList *tracks = new TrackList();
  while (child) 
    {
      if (xmlStrcasecmp(child->name, BAD_CAST "Artists") == 0) 
        parse(child, tracks);
      child = child->next;
    }
  return tracks;
}

Glib::ustring JamendoTrackListLoader::get_license_name(const char *data)
{
  Glib::ustring match = "http:\\/\\/creativecommons.org\\/licenses\\/";
  std::vector < Glib::ustring > a =
    Glib::Regex::split_simple (match, std::string(data));
  if (a.size () <= 1)
    return "";
  return a[1];
}
