//  Copyright (C) 2012, 2014 Ben Asselstine
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
//  02110-1301, USA.

#include <config.h>
#include <stdlib.h>
#include "license-image-cache.h"
#include "ucompose.hpp"

LicenseImageCache::LicenseImageCache()
{
  Glib::ustring parent = 
    String::ucompose ("%1/.cache/%2", getenv("HOME"), PACKAGE);
  dir = parent + "/license-images";
  if (Glib::file_test(dir,  Glib::FILE_TEST_IS_DIR))
    load(dir);
  else
    {
      guint32 mask = 0755;
      mkdir (parent.c_str(), mask);
      mkdir (dir.c_str(), mask);
    }
}

void LicenseImageCache::load(Glib::ustring d)
{
  Glib::ustring png = ".png";
  Glib::Dir dir(d);
  for (Glib::Dir::iterator i = dir.begin(), end = dir.end(); i != end; ++i)
    {
      Glib::ustring::size_type idx = (*i).rfind(png);
      if (idx != Glib::ustring::npos &&
          idx == (*i).length() - png.length())
        {
          std::string l = (*i).substr(0, idx);
          std::replace (l.begin(), l.end(), '|', '/');
          license_images[l] = Gdk::Pixbuf::create_from_file(d + "/" + (*i));
        }
    }
}

LicenseImageCache::~LicenseImageCache()
{
}

Glib::RefPtr<Gdk::Pixbuf> LicenseImageCache::load_license_image_from_data(CurlHelper::Msg &msg)
{
  Glib::RefPtr<Gio::MemoryInputStream> is = Gio::MemoryInputStream::create();
  is->add_data(msg.data, msg.len);
  return  Gdk::Pixbuf::create_from_stream(is);
}

void LicenseImageCache::add(Glib::ustring license, Glib::RefPtr<Gdk::Pixbuf> pixbuf)
{
  std::string l = license;
  std::replace (l.begin(), l.end(), '/', '|');
  pixbuf->save(String::ucompose("%1/%2.png", dir, l), "png");
  license_images[license] = pixbuf;
}

bool LicenseImageCache::on_license_image_downloaded(CurlHelper::Msg &msg, Glib::ustring track_id, Glib::ustring license, sigc::slot<void,Glib::ustring, Glib::ustring, Glib::RefPtr<Gdk::Pixbuf> > success, sigc::slot<void,Glib::ustring> failure)
{
  if (msg.response_code == 200)
    {
      Glib::RefPtr<Gdk::Pixbuf> image = load_license_image_from_data(msg);
      add(license, image);
      success(track_id, license, image);
      return true;
    }
  else
    {
      failure(track_id);
      return false;
    }
}

void LicenseImageCache::get_image(Glib::ustring track_id, Glib::ustring license, sigc::slot<void, Glib::ustring, Glib::ustring, Glib::RefPtr<Gdk::Pixbuf> > success, sigc::slot<void,Glib::ustring> failure)
{
  if (license_images.find(license) != license_images.end())
    {
      success(track_id, license, license_images[license]);
      return;
    }

  CurlHelper c(String::ucompose("http://%1/l/%2/88x31.png", 
                                "i.creativecommons.org", license));
  c.signal_received().connect
    (sigc::bind (sigc::mem_fun 
                 (this, &LicenseImageCache::on_license_image_downloaded),
                 track_id, license, success, failure));
  c.get();
}

