// Copyright (C) 2014 Ben Asselstine
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
//  02110-1301, USA.
#include <config.h>
#include "worker.h"
#include "curlhelper.h"
#include "gunziphelper.h"
#include "jamendo-track-list-loader.h"
#include "ucompose.hpp"
#include "track-list.h"

void Worker::on_gunzipped(Glib::ustring unzipped)
{
  JamendoTrackListLoader loader;
  tracks = loader.parse(unzipped.c_str());
  remove(unzipped.c_str());
  sig_done();
}

void Worker::on_file_downloaded(int response_code, Glib::ustring filename)
{
  if (response_code != 200)
    return;
  GunzipHelper helper(filename);
  helper.signal_unzipped().connect (sigc::mem_fun(*this, &Worker::on_gunzipped));
  helper.gunzip();
}

bool Worker::is_database_cached()
{
  Glib::ustring parent = 
    String::ucompose ("%1/.cache/%2", getenv("HOME"), PACKAGE);
  Glib::ustring dir = parent + "/track-database";

  if (Glib::file_test (dir +"/" +"dbdump_artistalbumtrack.xml.gz", Glib::FILE_TEST_EXISTS))
    return true;
  return false;
}

void Worker::run () 
{
  Glib::Mutex::Lock lock (mutex);
  if (stop)
    return;

  Glib::ustring parent = 
    String::ucompose ("%1/.cache/%2", getenv("HOME"), PACKAGE);
  mkdir (parent.c_str(), 0755);
  Glib::ustring dir = parent + "/track-database";
  mkdir (dir.c_str(), 0755);

  if (is_database_cached())
      on_file_downloaded(200, dir +"/" + "dbdump_artistalbumtrack.xml.gz");
  else
    {
      CurlHelper c("http://imgjam.com/data/dbdump_artistalbumtrack.xml.gz",
                   dir + "/" + "dbdump_artistalbumtrack.xml.gz");
      c.signal_downloaded().connect(sigc::mem_fun 
                                    (this, &Worker::on_file_downloaded));
      c.download();
    }
}
