// Copyright (C) 2014 Ben Asselstine
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
//  02110-1301, USA.

#ifndef WORKER_H
#define WORKER_H
#include <gtkmm.h>
#include "track-list.h"
class Worker 
{
public:

    Worker() : thread(0), stop(false), tracks(NULL) {}

    //Called to start the processing on the thread
    void start () {
      thread = Glib::Thread::create(sigc::mem_fun(*this, &Worker::run), true);
    }

    Glib::Dispatcher& signal_finished() {return sig_done;}

    // When shutting down, we need to stop the thread
    ~Worker() {
        {
          Glib::Mutex::Lock lock (mutex);
          stop = true;
        }
      if (thread)
        thread->join(); // Here we block to truly wait for the thread to complete
    }

    TrackList *get_tracks() {return tracks;}

    bool is_database_cached();

protected:
    // This is where the real work happens
    void run();

    Glib::Thread * thread;
    Glib::Mutex mutex;
    bool stop;
    TrackList *tracks;
    Glib::Dispatcher sig_done;

private:
    void on_file_downloaded(int response_code, Glib::ustring filename);
    void on_gunzipped(Glib::ustring unzipped);
};
#endif
